/*
    Copyright (C) 2002 Robert Muth <robert@muth.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; version 2 of June 1991.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program in form of the file COPYING; 
    if not, write to the 

    Free Software Foundation, Inc. <http://www.fsf.org>
    59 Temple Place, Suite 330, 
    Boston, MA 02111-1307  USA

    $Id: switch.H,v 1.3 2004/04/20 22:19:16 muth Exp $
*/

#ifndef SWITCH_H
#define SWITCH_H

typedef enum  
{ 
    SWITCH_TYPE_INVALID, 
    SWITCH_TYPE_BOOL, 
    SWITCH_TYPE_INT32, 
    SWITCH_TYPE_FLT32, 
    SWITCH_TYPE_STRING,
    SWITCH_TYPE_LAST
}SWITCH_TYPE;

typedef enum
{ 
    SWITCH_MODE_INVALID, 
    SWITCH_MODE_OVERWRITE, 
    SWITCH_MODE_ACCUMULATE,
    SWITCH_MODE_LAST
}SWITCH_MODE;

const string STRING_INVALID("*");

class VALUE
{
 public:
    VALUE *_next;
    const string _value;

    VALUE(const string& value) :
        _next(0),
        _value(value)
        {}

    BOOL ValBool() {return 0!=strtol(_value.c_str(), NULL,0); }
    string ValString() { return _value;}
    UINT32 ValInt32()  { return INT32(strtol(_value.c_str(), NULL,0));}
    FLT32 ValFlt32()  { return  FLT32(strtod(_value.c_str(), NULL));}
};


class SWITCH
{

    static SWITCH *_list;

private:
    
    SWITCH   *_next;
    const string   _family;
    const string   _name;
    const string   _purpose;    
    const SWITCH_TYPE _type;
    const SWITCH_MODE _mode;
    BOOL  _enabled;
    VALUE *_value;
    
public:
    // static members 

    static UINT32 SwitchNumber();

    static VOID FamilyDisable(const string& family);

    static VOID FamilyEnable(const string& family);

    static VOID SwitchDisable(const string& name);

    static VOID SwitchEnable(const string& name);

    static string SwitchSummary(BOOL enabled =1, BOOL listing =0);

    static SWITCH *SwitchFind(const string& name, BOOL enabled=1);

    // non static

    SWITCH_TYPE Type() const {return _type;}

    SWITCH_MODE Mode() const {return _mode;}

    string Family() const {return _family;}
    
    string Name() const {return _name;}

    UINT32 ValueNumber() const;
    
    VALUE *ValueGetByIndex(UINT32 index) const;

    VALUE *ValueAdd(const string& value);

    
    SWITCH(const string& name, const string& family, 
           SWITCH_TYPE  type, SWITCH_MODE mode, 
           const string& value, const string& purpose);

    BOOL ValueBool() const;
        
    INT32 ValueInt32() const;

    string ValueString() const;

    FLT32 ValueFlt32() const;

};

#endif








